@@ -6,6 +6,9 @@ fitting with the various models simultaneously.
 # Standard library imports
 import copy
 import inspect
+# import multiprocessing as mp
+# from multiprocessing import Process, Queue
+import os
 import pathlib
 import time
 
@@ -17,8 +20,14 @@ import pandas as pd
 from kipet.estimator_tools.multiple_experiments import MultipleExperimentsEstimator
 from kipet.main_modules.reaction_model import ReactionModel
 from kipet.general_settings.settings import Settings
+from kipet.general_settings.solver_settings import SolverSettings
 from kipet.general_settings.unit_base import UnitBase
+from kipet.estimator_tools.multiprocessing_kipet import Multiprocess
+from kipet.input_output.kipet_io import print_margin
 
+from kipet.model_tools.pyomo_model_tools import get_vars
+from kipet.model_tools.parameter_manager import ParameterManager, ModelParameter
+from kipet.general_settings.variable_names import VariableNames
 
 class ReactionSet:
     
@@ -79,10 +88,20 @@ class ReactionSet:
         self.global_parameters = None
         self.ub = UnitBase()
         
+        self.scale_variance = False
         self.file = pathlib.Path(inspect.stack()[1].filename)
         
         t = time.localtime()
-        self.timestamp = f'{t.tm_year}-{t.tm_mon:02}-{t.tm_mday:02}-{t.tm_hour:02}-{t.tm_min:02}-{t.tm_sec:02}'
+        file_add = f'{t.tm_year}-{t.tm_mon:02}-{t.tm_mday:02}-{t.tm_hour:02}-{t.tm_min:02}-{t.tm_sec:02}'
+        self.timestamp = f'{file_add}'
+        
+        self.solver_settings = SolverSettings()
+        if self.solver_settings.custom_solvers.ipopt is None:
+            if self.solver_settings.custom_solvers_lib['hsllib'] is not None:
+                self.settings.solver.update(**self.solver_settings.use_hsllib())
+            else:
+                print('No HSL libraries found for Ipopt used in CyIpopt')    
+        
 
     def __str__(self):
         
@@ -216,7 +235,8 @@ class ReactionSet:
                 #     new_kipet_model.model = None
                 
                 # if not copy_builder:
-                #     new_kipet_model.builder = TemplateBuilder()
+                from kipet.model_tools.template_builder import TemplateBuilder
+                new_kipet_model._builder = TemplateBuilder(name=new_kipet_model.name)
                     
                 # if not copy_components:
                 #     new_kipet_model.components = ComponentBlock()
@@ -295,12 +315,15 @@ class ReactionSet:
         if isinstance(model, ReactionModel):
             model.unit_base = self.ub
             self.reaction_models[model.name] = model
+            model.timestamp = self.timestamp
         else:
             raise ValueError("ReactionSet can only add ReactionModel instances.")
 
         return None
     
-    def run_opt(self, method='mee'):
+    def run_opt(self, method='mee', strategy='trust-region', parallel=False,
+                pre_solve=False, rh_method='pynumero', outer_locals=False,
+                variance_only=False):
         """This method will perform parameter fitting for all ReactionModels in
         the ReactionSet models attribute. If more than one ReactionModel instance
         is present, the MultipleExperimentEstiamtor is used to solve for the
@@ -315,38 +338,58 @@ class ReactionSet:
             The NSD method is not implemented at this time!
         
         :return: None
+        
         """
+        from kipet import __version__ as version_number
+        print_margin(f'KIPET v. {version_number}')
+        
+        print(f'# Date: {list(self.reaction_models.values())[0].timestamp}')
+        print(f'# File: {list(self.reaction_models.values())[0].file.stem}')
+        print(f'# ReactionModel instances: {", ".join(list(self.reaction_models.keys()))}')
+        print()
         
         if hasattr(self, "mee"):
             del self.mee
 
-        if len(self.reaction_models) > 1:
-            if method == "mee":
-                self._calculate_parameters()
-                self._create_multiple_experiments_estimator()
-                from kipet.input_output.kipet_io import Tee
+        if self.global_parameters is None:
+            self.global_parameters = list(self.all_params)
 
-                results_dir = pathlib.Path.cwd().joinpath(self.file.parent, 'results', f'{self.file.stem.lstrip("<").rstrip(">")}-{self.timestamp}')
-                if not results_dir.is_dir():
-                    results_dir.mkdir(parents=True)
+        from kipet.input_output.kipet_io import Tee
+        results_dir = pathlib.Path.cwd().joinpath(self.file.parent, 'results', f'{self.file.stem.lstrip("<").rstrip(">")}-{self.timestamp}')
+        if not results_dir.is_dir():
+            results_dir.mkdir(parents=True)
+        filename = results_dir.joinpath('log.txt') 
 
-                filename = results_dir.joinpath('log.txt') 
-
-                with Tee(filename): 
-                    self._run_full_model()
-            # elif method == 'nsd':
-            #     self._calculate_parameters()
-                # with Tee(f'log-{self.timestamp}.txt'):  
-            #         self._mee_nsd(strategy='ipopt')
+        #if len(self.reaction_models) > 1:
+        if method == "mee":
+            print('# ReactionSet: Starting multiple experiment estimator\n')
+            
+            if parallel:
+                #self.solve_mpire()
+                self.solve_mp()
             else:
-                raise ValueError("Not a valid method for optimization")
+                self._calculate_parameters(pre_solve=pre_solve, variance_only=variance_only)
                 
-            print('# KIPET procedure for multiple experiments finished\n')
-
+            self._create_multiple_experiments_estimator()
+            with Tee(filename): 
+                self._run_full_model()
+        
+        elif method == 'nsd':
+            print('# ReactionSet: Starting nested Schur decomposition estimator\n')
+           
+            with Tee(filename):  
+                self._mee_nsd(strategy=strategy, parallel=parallel, rh_method=rh_method, pre_solve=pre_solve, local_as_global=outer_locals)
+        
         else:
-            reaction_model = self.reaction_models[list(self.reaction_models.keys())[0]]
-            results = reaction_model.run_opt()
-            self.results[reaction_model.name] = results
+            raise ValueError("Not a valid method for optimization")
+                
+            print_margin('# KIPET procedure finished!')
+
+
+        # else:
+        #     reaction_model = self.reaction_models[list(self.reaction_models.keys())[0]]
+        #     results = reaction_model.run_opt()
+        #     self.results[reaction_model.name] = results
 
         return None
 
@@ -354,9 +397,9 @@ class ReactionSet:
         """A quick wrapper for MEE without big changes
         
         """
-        
         self.mee = MultipleExperimentsEstimator(self.reaction_models)
         self.mee.confidence_interval = self.settings.parameter_estimator.confidence
+        self.mee.free_params = self.define_free_parameters
 
         if "spectral" in self.data_types:
             self.settings.general.spectra_problem = True
@@ -365,19 +408,54 @@ class ReactionSet:
 
         self.mee.spectra_problem = self.settings.general.spectra_problem
 
-    def _calculate_parameters(self):
+    def _calculate_parameters(self, pre_solve=False, variance_only=False):
         """Uses the ReactionModel framework to calculate parameters instead of
         repeating this in the MEE
-
+        
         """
+        from kipet.estimator_tools.reduced_hessian_methods import add_warm_start_suffixes, update_warm_start
+        
         for name, model in self.reaction_models.items():
-            if not model._optimized:
-                model.run_opt()
+            if not model._optimized and not pre_solve:
+                
+                print_margin('ReactionSet - Model Preparation', sub_phrase=f'{model.name}')
+                
+                simulator = 'simulator'
+                estimator = 'p_estimator'
+                #model.simulate()
+                model._ve_set_up()
+                model._pe_set_up(solve=pre_solve)
+                vars_to_init = get_vars(model.p_model)
+                
+                #add_warm_start_suffixes(model.p_model)
+                model.p_model.ipopt_zL_in.update(model.p_model.ipopt_zL_out)
+                model.p_model.ipopt_zU_in.update(model.p_model.ipopt_zU_out)
+                
+                # for var in vars_to_init:
+                #     if hasattr(model.results_dict[simulator], var) and var != 'S':
+                #         getattr(model, estimator).initialize_from_trajectory(var, getattr(model.results_dict[simulator], var))
+                #     elif var == 'S' and hasattr(model.results_dict[simulator], 'S'):
+                #         getattr(model, estimator).initialize_from_trajectory(var, getattr(model.results_dict[simulator], var))
+                #     else:
+                #         pass
+                        #print(f'Variable: {var} is not updated')
+            
+            elif not model._optimized and pre_solve:
+                print(f'you are here for some reason: {pre_solve}')
+                model.run_opt(print_header=False)
             else:
-                print(f"Model {name} has already been optimized")
+                print(f"Model {model.name} has already been optimized or prepared")
+                
+            # for param, obj in model.p_model.P.items():
+            #     if model.parameters[param].fixed:
+            #         obj.fix()
+            
+        for name, model in self.reaction_models.items():
+            model.min_variance = self.min_variance
 
         return None
 
+
     def _run_full_model(self):
         """Runs the MEE after everything has been set up"""
         
@@ -395,7 +473,7 @@ class ReactionSet:
 
         return results
 
-    def _mee_nsd(self, strategy='ipopt'):
+    def _mee_nsd(self, strategy='ipopt', parallel=False, rh_method='pynumero', pre_solve=False, local_as_global=False):
         """Performs the NSD on the multiple ReactionModel instances
 
         :parameter str strategy: Method used to control the outer problem
@@ -405,31 +483,64 @@ class ReactionSet:
         :rtype: dict
 
         """
+        from kipet.estimator_tools.nested_schur_decomposition import NSD
         kwargs = {'kipet': True,
-                  'objective_multiplier': 1
+                  'objective_multiplier': 1,
                   }
+        
+        print(self.settings)
+        
+        if self.scale_variance:
+            for model in self.reaction_models.values():
+                model.settings.parameter_estimator.scale_variance = True
+                #if self.global_variance:
+                model.min_variance_global = self.min_variance
+                
+        self._calculate_parameters(pre_solve=pre_solve)
 
-        if self.global_parameters is not None:
-            global_parameters = self.global_parameters
-        else:
-            global_parameters = self.all_params
+        self.param_manager = ParameterManager(self.reaction_models)
 
         self.nsd = NSD(self.reaction_models,
-                        strategy=strategy,
-                        global_parameters=global_parameters,
-                        kwargs=kwargs)
+                       strategy=strategy,
+                       global_parameters=self.global_parameters,
+                       parallel=parallel,
+                       kwargs=kwargs,
+                       rh_method=rh_method,
+                       rs_min_variance=self.min_variance,
+                       all_params=self.param_manager,
+                       #all_params=self.rs_params,
+                       free_params=self.define_free_parameters,
+                       local_as_global=local_as_global)
 
         #print(self.nsd.d_init)
-
         results = self.nsd.run_opt()
 
         self.results = results
-        for key, results_obj in self.results.items():
-            results_obj.file_dir = self.settings.general.charts_directory
+        # for key, results_obj in self.results.items():
+        #     results_obj.file_dir = self.settings.general.charts_directory
 
         return results
     
 
+    @property
+    def rs_params(self):
+        
+        parameter_list = []
+        for i, (name, model) in enumerate(self.reaction_models.items()):
+            for param in model.parameters:
+                if param.fixed:
+                    continue
+                
+                p_obj = ModelParameter(
+                    param.name, 
+                    model.name, 
+                    param.name in self.global_parameters,
+                    param)
+                parameter_list.append(p_obj)
+                
+        return parameter_list
+        
+
     @property
     def all_params(self):
         """
@@ -442,7 +553,7 @@ class ReactionSet:
         set_of_all_model_params = set()
         for name, model in self.reaction_models.items():
             set_of_all_model_params = set_of_all_model_params.union(
-                model.parameters.names
+                model.parameters.get_match('fixed', False)
             )
 
         return set_of_all_model_params
@@ -461,6 +572,9 @@ class ReactionSet:
             for dataset_name, dataset in kipet_model.datasets.datasets.items():
                 data_types.add(dataset.category)
                 
+            if kipet_model.spectra is not None and kipet_model.spectra.data.size > 0:
+                data_types.add('spectral')
+                
         return data_types
     
     
@@ -481,6 +595,25 @@ class ReactionSet:
             
         return df_param
     
+    @property
+    def min_variance(self):
+        """Finds the lowest variance in the set of ReactionModel instances
+        to better scale the objective function
+        
+        TODO: Take into account the scale of the numerator (the concentrations)
+        
+        """
+        var = 1
+        
+        if self.scale_variance:
+            for i, model in self.reaction_models.items():
+                if not model.spectra:
+                    var = min(min([v for v in model.components.variances.values()]), var)
+                else:
+                    var = min(min([v for v in model.results_dict['v_estimator'].sigma_sq.values()]), var)
+    
+        return var
+    
     def plot(self, var=None):
         """Plot method for ReactionSet
         
@@ -505,7 +638,103 @@ class ReactionSet:
             
         """
         from kipet.visuals.reports import Report
-        for reaction in self.reaction_models.values():
+        for name, reaction in self.reaction_models.items():
+            print(f'# ReactionSet: Making plots for {name}')
             reaction.plot()
+            
         self.report_object = Report(list(self.reaction_models.values()))
-        self.report_object.generate_report()
\ No newline at end of file
+        self.report_object.generate_report()
+        
+    def func(self, q, i):
+        print('starting')
+        print('process_id', os.getpid())
+        
+        model_to_solve = list(self.reaction_models.values())[i - 1]
+        data = self.solve(model_to_solve)
+        q.put(data)
+        print('all done')
+
+    def solve(self, model):
+        """Uses the ReactionModel framework to calculate parameters instead of
+        repeating this in the MEE
+    
+        # Add mp here
+    
+        """
+        if not model._optimized:
+            model.run_opt()
+            print('Model has been optimized')
+        else:
+            print(f"Model {model.name} has already been optimized")
+    
+    
+        attr_list = ['name', 'results_dict']
+    
+        model_dict = {}
+        for attr in attr_list:
+            model_dict[attr] = getattr(model, attr)
+    
+        return model_dict['results_dict']
+    
+    def solve_mp(self):
+    
+        from multiprocessing import set_start_method, cpu_count
+        from kipet.model_tools.pyomo_model_tools import get_vars
+        import platform
+        
+        if platform.system() == 'Darwin':
+            try:
+                set_start_method('fork')
+                print('# Changing Multiprocessing start method to "fork"')
+            except:
+                print('# Multiprocessing start method already fixed')   
+    
+        mp = Multiprocess(self.func)
+        data = mp(num_processes = len(self.reaction_models)) #cpu_count()))
+        
+        self.mp_results = data
+        estimator = 'p_estimator'
+        
+        for i, model in enumerate(self.reaction_models.values()):
+            model._pe_set_up(solve=False)
+            
+            setattr(model, 'results_dict', self.mp_results[i + 1])
+            setattr(model, 'results', self.mp_results[i + 1][estimator])
+            vars_to_init = get_vars(model.p_model)
+            #print(model.results_dict[estimator])
+            #print(vars_to_init)
+            
+            for var in vars_to_init:
+                if hasattr(model.results_dict[estimator], var) and var != 'S':
+                    getattr(model, estimator).initialize_from_trajectory(var, getattr(model.results_dict[estimator], var))
+                elif var == 'S' and hasattr(model.results_dict[estimator], 'S'):
+                    getattr(model, estimator).initialize_from_trajectory(var, getattr(model.results_dict[estimator], var))
+                else:
+                    pass
+                    #print(f'Variable: {var} is not updated')
+            
+        return None
+    
+    @property
+    def define_free_parameters(self):
+        """Identifies the free variables used in MEE and NSD methods
+           
+        :return: list of free variable names
+        
+        """
+        free_parameters = []
+        global_accounted = {}
+        
+        for p_obj in self.rs_params:
+            
+            if p_obj.parameter_object.fixed:
+                continue
+            
+            if p_obj.is_global and p_obj.name in global_accounted:
+                continue
+            
+            else:
+                free_parameters.append(p_obj)
+                global_accounted[p_obj.name] = True
+        
+        return free_parameters
